@@ -8,6 +8,7 @@
 #include <string.h>
 #include <iostream>
 #include <sstream>
+#include <string_view>
 
 #include <zypp/ZYppFactory.h>
 #include <zypp/base/LogTools.h>
@@ -60,6 +61,112 @@ namespace {
 
   std::string resolverForceResolutionHintVendorChange()
   { return _resolverForceResolutionHint( &zypp::Resolver::allowVendorChange ); }
+
+
+  // --------------------------------------------------------------------------
+  struct SolvableSummaryItemFormater
+  {
+    SolvableSummaryItemFormater( ansi::Color color_r, bool withKind_r = false )
+    : _configHighlight { Zypper::instance().config().color_pkglistHighlight }
+    , _configAttr { Zypper::instance().config().color_pkglistHighlightAttribute }
+    , _contextColor { color_r }
+    , _withKind { withKind_r }
+    {
+      if ( _withKind )
+        _extractFn = &SolvableSummaryItemFormater::extractIdent;
+      if ( _configHighlight )
+        _formatFn = &SolvableSummaryItemFormater::formatHighlightFirst;
+      else if ( indeterminate(_configHighlight) )
+        _formatFn = &SolvableSummaryItemFormater::formatHighlightIfChanged;
+    }
+
+    template<class TSolv>
+    std::ostream & operator()( std::ostream & str, const sat::SolvableType<TSolv> & slv_r ) const
+    { return operator()( str, slv_r.satSolvable() ); }
+
+    std::ostream & operator()( std::ostream & str, sat::Solvable slv_r ) const
+    { return quoteIf( str, (this->*_extractFn)( slv_r ) ); }
+
+ private:
+    std::ostream & quoteIf( std::ostream & str, const std::string & txt_r ) const
+    {
+      if ( txt_r.find_first_of( " " ) == std::string::npos )
+        return (this->*_formatFn)( str, txt_r );
+      else {
+        static const HIGHLIGHTString quoteCh( "\"" );
+        return (this->*_formatFn)( str << quoteCh, txt_r ) << quoteCh;
+      }
+    }
+
+  private:
+    using ExtractFn = std::string (SolvableSummaryItemFormater::*)( sat::Solvable slv_r ) const;
+    using FormatFn = std::ostream & (SolvableSummaryItemFormater::*)( std::ostream &, const std::string & ) const;
+
+  private:
+    std::string extractName( sat::Solvable slv_r ) const
+    { return slv_r.name(); }
+
+    std::string extractIdent( sat::Solvable slv_r ) const
+    { return slv_r.ident().asString(); }
+
+  private:
+    std::ostream & formatPlain( std::ostream & str, const std::string & txt_r ) const
+    { return str << txt_r; }
+
+    std::ostream & formatHighlightFirst( std::ostream & str, const std::string & txt_r ) const
+    {
+      std::string_view txt { txt_r };
+      return str << ( _contextColor << _configAttr << txt.substr( 0, 1 ) ) << txt.substr( 1 ); }
+
+    std::ostream & formatHighlightIfChanged( std::ostream & str, const std::string & txt_r ) const
+    {
+      if ( txt_r[0] != _first ) {
+        _first = txt_r[0];
+        return formatHighlightFirst( str, txt_r );
+      }
+      return formatPlain( str, txt_r );
+    }
+
+   private:
+    TriBool _configHighlight;   ///< always/never or on a changed first char
+    ansi::Color _configAttr;    ///< highlight attribute from config
+    ansi::Color _contextColor;  ///< list color may be overwritten by _configAttr
+    bool _withKind = false;     ///< prefix names by kind
+
+    mutable char _first = '\0'; ///< for changed first char highlighting
+
+    ExtractFn _extractFn = &SolvableSummaryItemFormater::extractName;
+    FormatFn _formatFn = &SolvableSummaryItemFormater::formatPlain;
+  };
+
+  template <typename TContainer>  // sat::SolvableType
+  std::ostream & writeSolvableList( std::ostream & str, const TContainer & container_r, ansi::Color color_r, bool withKind_r=false )
+  {
+    constexpr unsigned indent = 2;
+    SolvableSummaryItemFormater format { color_r, withKind_r };
+
+    std::ostringstream s;
+    for ( const auto & slv : container_r ) {
+      format( s, slv ) << " "; // don't mind trailing WS, it's eaten in mbs_write_wrapped
+    }
+    mbs_write_wrapped( str, s.str(), indent, get_screen_width() );
+    return str << endl;
+  }
+
+  template <typename TContainer>  // std::pair<sat::SolvableType,std::string>
+  std::ostream & writeSolvableDetailsList( std::ostream & str, const TContainer & container_r, ansi::Color color_r, bool withKind_r=false )
+  {
+    constexpr unsigned indent = 2;
+    SolvableSummaryItemFormater format { color_r, withKind_r };
+
+
+    std::ostringstream s;
+    for ( const auto & pair : container_r ) {
+      format( s, pair.first ) << " " << pair.second << endl;
+    }
+    mbs_write_wrapped( str, s.str(), indent, get_screen_width() );
+    return str << endl;
+  }
 } // namespace
 
 // --------------------------------------------------------------------------
@@ -74,11 +181,13 @@ bool Summary::ResPairNameCompare::operator()( const ResPair & p1, const ResPair
 
 // --------------------------------------------------------------------------
 
-Summary::Summary( const ResPool & pool, const ViewOptions options )
-: _viewop( options )
+Summary::Summary( const ResPool & pool, SummaryHints summaryHints, const ViewOptions options )
+: _summaryHints { std::move(summaryHints) }
+, _viewop( options )
 , _wrap_width( 80 )
 , _force_no_color( false )
 , _download_only( false )
+, _dryRun( false )
 {
   readPool( pool );
 }
@@ -110,7 +219,8 @@ void Summary::readPool( const ResPool & pool )
 
   _todownload = ByteCount();
   _incache = ByteCount();
-  _inst_size_change = ByteCount();
+  _inst_size_install = ByteCount();
+  _inst_size_remove = ByteCount();
 
   _ctc.clear();
 
@@ -205,8 +315,8 @@ void Summary::readPool( const ResPool & pool )
     for_( resit, it->second.begin(), it->second.end() )
     {
       ResObject::constPtr res(*resit);
-
       Package::constPtr pkg = asKind<Package>(res);
+
       if ( pkg )
       {
         switch ( pkg->vendorSupport() )
@@ -220,6 +330,8 @@ void Summary::readPool( const ResPool & pool )
           case VendorSupportACC:
             _supportNeedACC[res->kind()].insert( ResPair( nullptr, res ) );
             break;
+          case VendorSupportSuperseded:
+            _supportSuperseded[res->kind()].insert( ResPair( nullptr, res ) );
           default:
             // L1, L2 or L3 support are not reported
             break;
@@ -273,7 +385,8 @@ void Summary::readPool( const ResPool & pool )
               _tochangevendor[res->kind()].insert( rp );
           }
 
-          _inst_size_change += res->installSize() - (*rmit)->installSize();
+          _inst_size_install += res->installSize();
+          _inst_size_remove += (*rmit)->installSize();
 
           // this turned out to be an upgrade/downgrade
           to_be_removed[res->kind()].erase( *rmit );
@@ -285,7 +398,7 @@ void Summary::readPool( const ResPool & pool )
       if ( !upgrade_downgrade )
       {
         _toinstall[res->kind()].insert( ResPair( nullptr, res ) );
-        _inst_size_change += res->installSize();
+        _inst_size_install += res->installSize();
       }
 
       if ( pkg && pkg->isCached() )
@@ -312,7 +425,7 @@ void Summary::readPool( const ResPool & pool )
           _toremove_by_solver = true;
       }*/
       _toremove[it->first].insert( ResPair( nullptr, *resit ) );
-      _inst_size_change -= (*resit)->installSize();
+      _inst_size_remove += (*resit)->installSize();
     }
 
   m.elapsed();
@@ -1294,6 +1407,46 @@ void Summary::writeSupportNeedACC( std::ostream & out )
   }
 }
 
+void Summary::writeSupportSuperseded( std::ostream & out )
+{
+  auto highlightColor = ColorContext::HIGHLIGHT;
+  auto supersedingAsString = []( Package::constPtr package_r ) -> std::string {
+    const std::pair<std::vector<IdString>,std::vector<std::string>> & supersededByItems { package_r->supersededByItems() };
+    const std::vector<IdString> & superseding { supersededByItems.first };
+    str::Str ret;
+    ret << _("was superseded by");
+    if ( not superseding.empty() )  // names resolved to existing packages
+      dumpRange( ret.stream(), superseding.begin(), superseding.end(), " ", "", ", ", "", "" );
+    else  // print at least the plain names
+      dumpRange( ret.stream(), supersededByItems.second.begin(), supersededByItems.second.end(), " ", "", ", ", "", "" );
+    return ret.str();
+  };
+
+  for_( it, _supportSuperseded.begin(), _supportSuperseded.end() )
+  {
+    // we only look at vendor support in packages
+    if ( it->first == ResKind::package ) {
+      std::string label = PL_(
+        "The following package was discontinued and has been superseded by a new package with a different name.",
+        "The following %d packages were discontinued and have been superseded by a new packages with different names.",
+        it->second.size() );
+      label = str::form( label.c_str(), it->second.size() );
+      std::string labelHint = PL_(
+        "The successor package should be installed as soon as possible to receive continued support:",
+        "The successor packages should be installed as soon as possible to receive continued support:",
+        it->second.size() );
+      out << endl << ( highlightColor << label << endl << labelHint ) << endl;
+
+      std::vector<std::pair<sat::Solvable,std::string>> itemsdetails;
+      for ( const auto & pair : it->second ) {
+        Package::constPtr package = pair.second->asKind<Package>();
+        itemsdetails.push_back( std::make_pair( package->satSolvable(), supersedingAsString( package ) ) );
+      }
+      writeSolvableDetailsList( out, itemsdetails, highlightColor );
+    }
+  }
+}
+
 // --------------------------------------------------------------------------
 
 void Summary::writeNotUpdated( std::ostream & out )
@@ -1404,6 +1557,15 @@ void Summary::writeRebootNeeded( std::ostream & out )
   checkAndPrintRebootNeeded( ResKind::package );
 }
 
+void Summary::writeAutoResolved( std::ostream & out )
+{
+  if ( _summaryHints.haveSkippedPatches() ) {
+    std::string label { _("Skipped needed patches which do not apply without conflict:") };
+    out << endl << ( ColorContext::MSG_WARNING << label << " [-skip-not-applicable-patches]" ) << endl;
+    writeSolvableList( out, _summaryHints.skippedPatchesSet(), ColorContext::MSG_WARNING );
+  }
+}
+
 // --------------------------------------------------------------------------
 
 void Summary::writeDownloadAndInstalledSizeSummary( std::ostream & out )
@@ -1411,33 +1573,50 @@ void Summary::writeDownloadAndInstalledSizeSummary( std::ostream & out )
   if ( !_inst_pkg_total && _toremove.empty() )
     return; // nothing to do, keep silent
 
-  // download size info
-  std::ostringstream s;
-  if ( _todownload || _incache )
-    s << str::Format(_("Overall download size: %1%. Already cached: %2%.")) % _todownload % _incache << " ";
+  auto fmtBC = []( ByteCount bc_r, const char * sign = " " ) {
+    return sign + bc_r.asString( 7 );
+  };
 
-  if ( _download_only )
-    s << _("Download only.");
-  else
-  {
-    // installed size change info
-    if ( _inst_size_change > 0 )
-      // TrasnlatorExplanation %s will be substituted by a byte count e.g. 212 K
-      s << str::Format(_("After the operation, additional %s will be used.")) % _inst_size_change.asString( 0 , 1, 1 );
-    else if ( _inst_size_change == 0 )
-      s << _("No additional space will be used or freed after the operation.");
-    else
-    {
-      // get the absolute size
-      ByteCount abs;
-      abs = (-_inst_size_change);
-      // TrasnlatorExplanation %s will be substituted by a byte count e.g. 212 K
-      s << str::Format(_("After the operation, %s will be freed.")) % abs.asString( 0, 1, 1 );
+  out << endl;
+
+  if ( _todownload || _incache ) {
+    //   Package download size:   99.2 MiB
+    // or
+    //   Package download size:
+    //                 |     105.7 MiB  overall download size
+    //       99.2 MiB  |  -    6.4 MiB  already in cache
+    const auto & tagstr = _("Package download size:");
+    if ( not _incache ) {
+      out << tagstr << fmtBC( _todownload+_incache ) << endl;
+    } else {
+      out << tagstr << endl;
+      Table t;
+      t.lineStyle( TableLineStyle::none );
+      // translator: rendered as "105.7 MiB  overall download size"
+      t << ( TableRow() << ""                   <<"|"<< fmtBC( _todownload+_incache ) << _("overall package size") );
+      // translator: rendered as "  6.4 MiB  already in cache"
+      t << ( TableRow() << fmtBC( _todownload ) <<"|"<< fmtBC( _incache, "-" ) << _("already in cache") );
+      out << t; // NL terminated
     }
+    out << endl;
   }
 
-  mbs_write_wrapped( out, s.str(), 0, _wrap_width );
-  out << endl;
+  if ( _download_only ) {
+    out << _("Download only.") << endl;
+  } else {
+    // Package install size change:
+    //               |     349.1 MiB  required by to be installed packages
+    //      1.9 MiB  |  -  347.3 MiB  released by to be removed packages
+    out << _("Package install size change:") << endl;
+    ByteCount inst_diff = _inst_size_install-_inst_size_remove;
+    Table t;
+    t.lineStyle( TableLineStyle::none );
+    // translator: rendered as "349.1 MiB  required by to be installed packages"
+    t << ( TableRow() << ""                 <<"|"<< fmtBC( _inst_size_install ) << _("required by to be installed packages") );
+    // translator: rendered as "347.3 MiB  released by to be removed packages"
+    t << ( TableRow() << fmtBC( inst_diff ) <<"|"<< fmtBC( _inst_size_remove, "-" ) << _("released by to be removed packages") );
+    out << t; // NL terminated
+  }
 }
 
 void Summary::writePackageCounts( std::ostream & out )
@@ -1621,11 +1800,13 @@ void Summary::dumpTo( std::ostream & out )
   writeChangedVendor(out);
   writeNewlyInstalled( out );
   writeRemoved( out );
+  writeAutoResolved( out );
   if ( _viewop & SHOW_UNSUPPORTED )
   {
     writeSupportUnknown( out );
     writeSupportUnsupported( out );
     writeSupportNeedACC( out );
+    writeSupportSuperseded( out );
   }
   writeRebootNeeded( out );
   writePackageCounts( out );
@@ -1636,7 +1817,15 @@ void Summary::dumpTo( std::ostream & out )
     Zypper::instance().out().notePar( 4, _("Package manager restart required. (Run this command once again after the update stack got updated)") );
   }
   if ( showNeedRebootHInt() )
-  {   Zypper::instance().out().notePar( 4, _("System reboot required.") ); }
+  {
+    if ( _download_only || _dryRun ) {
+      std::string_view opt { _dryRun ? "--dry-run" : "--download-only" };
+      // translator: %1% is an option string like  "--dry-run"
+      Zypper::instance().out().notePar( 4, str::Format( _("%1% is set, otherwise a system reboot would be required.") ) % opt );
+    } else {
+      Zypper::instance().out().notePar( 4, _("System reboot required.") );
+    }
+  }
 
   if ( !_ctc.empty() )
   {
@@ -1703,10 +1892,13 @@ void Summary::dumpAsXmlTo( std::ostream & out )
   const auto & iter = _toremove.find( ResKind::package );
   if ( iter != _toremove.end() )
     pkgchanged += iter->second.size();
+  zypp::ByteCount _inst_size_change = _inst_size_install - _inst_size_remove;
 
   out << "<install-summary";
   out << " download-size=\"" << ((ByteCount::SizeType)_todownload) << "\"";
   out << " space-usage-diff=\"" << ((ByteCount::SizeType)_inst_size_change) << "\"";
+  out << " space-usage-installed=\"" << ((ByteCount::SizeType)_inst_size_install) << "\"";
+  out << " space-usage-removed=\"" << ((ByteCount::SizeType)_inst_size_remove) << "\"";
   out << " packages-to-change=\"" << pkgchanged << "\"";	// bsc#1102429: CaaSP requires it to detect 'nothing to do'
   out << " need-restart=\"" << showNeedRestartHint() << "\"";	// bsc#1188435: show need reboot/restart hints
   out << " need-reboot=\"" << showNeedRebootHInt() << "\"";	// -"-
