@@ -63,50 +63,75 @@ private:
   Pathname _watch;
 };
 
+///////////////////////////////////////////////////////////////////
+namespace {
+  inline ColorString tagProblem() {
+    // translators: meaning 'dependency problem' found during solving
+    return MSG_WARNINGString(_("Problem: "));
+  }
+  inline ColorString tagProblem( unsigned i ) {
+    std::string s { std::to_string(i)+": " };
+    // TODO: fixup .po files "Problem: %s" to fit here
+    // TranslatorExplanation %d is the problem number
+    return MSG_WARNINGString(str::Format(_("Problem: %s")) % s);
+  }
+  inline ColorString tagSolution( unsigned i ) {
+    // TranslatorExplanation %d is the solution number
+    return HIGHLIGHTString(str::Format(_(" Solution %d: ")) % i);
+  }
+
+  // Print a problems description (optionally verbose)
+  inline std::ostream & dumpProblem( std::ostream & str, unsigned nr, const ResolverProblem & problem_r, bool verbose_r = false )
+  {
+    MLSep sep;
+    // description() is one out of all in completeProblemInfo()
+    // picked by the resolver to represent the problem.
+    if ( not verbose_r ) {
+      str << sep << tagProblem(nr) << problem_r.description();
+    } else {
+      str << sep << tagProblem(nr) << _("Detailed information: ");
+      for ( const std::string & pRule : problem_r.completeProblemInfo() ) {
+        str << sep << "- " << pRule;
+      }
+    }
+    const std::string & details { problem_r.details() };
+    if ( not details.empty() )
+      str << endl << details << endl;
+    return str;
+  }
+
+  // Print a single problem solution
+  inline std::ostream & dumpSolution( std::ostream & str, unsigned nr, const ProblemSolution & solution_r )
+  {
+    MLSep sep;
+    str << sep << tagSolution(nr) << solution_r.description();
+    const std::string & details { solution_r.details() };
+    if ( not details.empty() )
+      str << sep << indent( details, 2 );
+    return str;
+  }
+
+  // Print a problems solutions
+  inline std::ostream & dumpSolutions( std::ostream & str, const ProblemSolutionList & solutions_r )
+  {
+    MLSep sep;
+    unsigned n = 0;
+    for ( const auto & solPtr : solutions_r )
+      dumpSolution( str << sep, ++n, *solPtr );
+    return str;
+  }
+} // namespace
+///////////////////////////////////////////////////////////////////
 
 //! @return true to retry solving now,
 //!         false to cancel,
 //!         indeterminate to continue
-static TriBool show_problem( Zypper & zypper, const ResolverProblem & prob, ProblemSolutionList & todo )
+static TriBool show_problem( Zypper & zypper, unsigned nr, const ResolverProblem & prob, ProblemSolutionList & todo )
 {
-  std::string tmp;
-  std::ostringstream desc_stm;
-  // translators: meaning 'dependency problem' found during solving
-  desc_stm << _("Problem: ") << prob.description() << endl;
-  tmp = prob.details();
-  if ( !tmp.empty() )
-    desc_stm << "  " << tmp << endl;
-
-  std::ostringstream desc_ext_stm;
-  desc_ext_stm << _("Detailed information: ") << endl;
-  for ( const std::string &pRule : prob.completeProblemInfo() ) {
-    desc_ext_stm << "\t" << pRule << endl;
-  }
-
-  const ProblemSolutionList & solutions = prob.solutions();
-
-  unsigned n = 0;
-  std::ostringstream solution_stm;
-  for ( const auto & solPtr : solutions )
-  {
-    ++n;
-    // TranslatorExplanation %d is the solution number
-    solution_stm << str::Format(_(" Solution %d: ")) % n << solPtr->description() << endl;
-    tmp = solPtr->details ();
-    if ( !tmp.empty () )
-      solution_stm << indent( tmp, 2 ) << endl;
-  }
-
+  const ProblemSolutionList & solutions { prob.solutions() };
   unsigned problem_count = God->resolver()->problems().size();
   unsigned solution_count = solutions.size();
 
-  // without solutions, its useless to prompt
-  if ( solutions.empty() )
-  {
-    zypper.out().error( desc_stm.str() + solution_stm.str() );
-    return false;
-  }
-
   std::string prompt_text;
   if ( problem_count > 1 )
     prompt_text = PL_(
@@ -179,10 +204,8 @@ static TriBool show_problem( Zypper & zypper, const ResolverProblem & prob, Prob
   while ( true ) {
 
     std::ostringstream fulltext;
-    fulltext << desc_stm.str();
-    if ( showExtInfo )
-      fulltext << desc_ext_stm.str();
-    fulltext << solution_stm.str();
+    dumpProblem( fulltext, nr, prob, showExtInfo ) << endl;
+    dumpSolutions( fulltext, solutions ) << endl;
 
     zypper.out().prompt( PROMPT_DEP_RESOLVE, prompt_text, popts, fulltext.str() );
     reply = get_prompt_reply( zypper, PROMPT_DEP_RESOLVE, popts );
@@ -213,16 +236,12 @@ static TriBool show_problem( Zypper & zypper, const ResolverProblem & prob, Prob
 }
 
 // return true to retry solving, false to cancel transaction
-static bool show_problems( Zypper & zypper )
+static bool show_problems( Zypper & zypper, SolveAndCommitPolicy & policy )
 {
+  //! \todo handle resolver problems caused by --capability mode arguments specially to give proper output (bnc #337007)
   Resolver_Ptr resolver = getZYpp()->resolver();
   const ResolverProblemList & rproblems( resolver->problems() );
-
-  // display the number of problems
-  if ( rproblems.size() > 1 )
-    zypper.out().info( str::Format(PL_("%d Problem:", "%d Problems:", rproblems.size())) % rproblems.size() );
-  else if ( rproblems.empty() )
-  {
+  if ( rproblems.empty() ) {
     // should not happen! If solve() failed at least one problem must be set!
     INT << "Resolver failed but reported no problems." << endl;
     zypper.out().error(_("Resolver failed but reported no problems.") );
@@ -230,32 +249,77 @@ static bool show_problems( Zypper & zypper )
     return false; // will cancel the transaction
   }
 
-  // for many problems, list them shortly first
-  //! \todo handle resolver problems caused by --capability mode arguments specially to give proper output (bnc #337007)
-  if (rproblems.size() > 1)
-  {
-    for ( const auto & probPtr : rproblems )
-      zypper.out().info( str::Format(_("Problem: %s")) % probPtr->description() );
+  ProblemSolutionList pendingSolutions; // List of solutions to apply before retrying.
+
+  bool printAbstract = rproblems.size() > 1;              // print all problems in advance, not just auto-resolvable ones
+  bool mayAutoResolve = policy.skipNotApplicablePatches(); // by now just NotApplicablePatches
+
+  if ( printAbstract ) {
+    // display the number of problems
+    zypper.out().info( MSG_WARNINGString(str::Format(PL_("%d Problem:", "%d Problems:", rproblems.size())) % rproblems.size()).str() );
   }
 
+  // If there are multiple problems or if any kind of auto-resolution is enabled,
+  // go through the list of problems and list them shortly and/or try to auto-resolve.
+  if ( printAbstract || mayAutoResolve ) {
+    unsigned p = 0;
+    for ( const auto & probPtr : rproblems ) {
+      ++p;
+
+      if ( printAbstract ) {
+        dumpProblem( zypper.out().info(), p, *probPtr );
+      }
+
+      if ( mayAutoResolve ) {
+        unsigned s = 0;
+        for ( const auto & solPtr : probPtr->solutions() )
+        {
+          if ( policy.skipNotApplicablePatches() ) {
+            std::optional<std::set<PoolItem>> items { solPtr->getIfSkipsPatchesOnly() };
+            if ( items ) {
+              pendingSolutions.push_back( solPtr );
+              policy.summaryHints.skippedPatchesSet().merge( std::move(*items) );
+              items.reset();  // moved out
+              if ( not printAbstract ) { // was not printed before
+                dumpProblem( zypper.out().info(), p, *probPtr );
+              } else {
+                // Once we auto-resolved at least one problem, we will solve again before
+                // asking the user anything. So we stop printing any summary and list only
+                // auto-resolved problems.
+                printAbstract = false;
+              }
+              // translator: The tag says that the preceding dependency problem is auto-resolved by applying the following solution
+              zypper.out().infoLRHint( HIGHLIGHTString(indent( _("Autoresolve:"), 1 )).str(), "--skip-not-applicable-patches" );
+              dumpSolution( zypper.out().info(), ++s, *solPtr );
+            }
+          }
+        }
+      }
+    }
+  }
+  // NOTE: summaryHints for auto-resolved pendingSolutions were already collected above,
+  // knowing that the solutions will actually be applied below.
   bool retry = true;
-  ProblemSolutionList todo;
-  // now list all problems with solution proposals
-  for ( const auto & probPtr : rproblems )
-  {
-    zypper.out().info( "", Out::NORMAL, Out::TYPE_NORMAL );	// visual separator
-    TriBool stopnow = show_problem( zypper, *probPtr, todo );
-    if ( !indeterminate( stopnow ) )
+
+  if ( pendingSolutions.empty() ) {
+    // If nothing's been auto applied so far: ask!
+    unsigned n = 0;
+    // now list all problems with solution proposals
+    for ( const auto & probPtr : rproblems )
     {
-      retry = bool(stopnow);
-      break;
+      zypper.out().info( "", Out::NORMAL, Out::TYPE_NORMAL );	// visual separator
+      TriBool stopnow = show_problem( zypper, ++n, *probPtr, pendingSolutions );
+      if ( not indeterminate(stopnow) ) {
+        retry = bool(stopnow);
+        break;
+      }
     }
   }
 
-  if ( retry )
-  {
+  if ( retry ) {
+    zypper.out().gap();
     zypper.out().info(_("Resolving dependencies...") );
-    resolver->applySolutions( todo );
+    resolver->applySolutions( pendingSolutions );
   }
   return retry;
 }
@@ -384,6 +448,8 @@ static void set_solver_flags( Zypper & zypper )
     if ( !indeterminate( solverSettings._allowNameChange ) ) God->resolver()->dupSetAllowNameChange( bool(solverSettings._allowNameChange) );
     if ( !indeterminate( solverSettings._allowArchChange ) ) God->resolver()->dupSetAllowArchChange( bool(solverSettings._allowArchChange) );
     if ( !indeterminate( solverSettings._allowVendorChange ) ) God->resolver()->dupSetAllowVendorChange( bool(solverSettings._allowVendorChange) );
+    // by now libsolv supports it in distupgrade only:
+    if ( !indeterminate( solverSettings._removeOrphaned ) ) God->resolver()->setRemoveOrphaned( bool(solverSettings._removeOrphaned) );
   }
   else
   {
@@ -504,6 +570,12 @@ bool SolveAndCommitPolicy::forceCommit() const
 SolveAndCommitPolicy &SolveAndCommitPolicy::forceCommit(bool enable)
 { _forceCommit = enable; return *this; }
 
+bool SolveAndCommitPolicy::skipNotApplicablePatches() const
+{ return _skipNotApplicablePatches; }
+
+SolveAndCommitPolicy & SolveAndCommitPolicy::skipNotApplicablePatches( bool enable )
+{ _skipNotApplicablePatches = enable; return *this; }
+
 const Summary::ViewOptions &SolveAndCommitPolicy::summaryOptions() const
 { return _summaryOptions; }
 
@@ -628,6 +700,7 @@ void solve_and_commit ( Zypper &zypper, SolveAndCommitPolicy policy )
 {
   bool need_another_solver_run = true;
   bool dryRunEtc = policy.zyppCommitPolicy().dryRun() || ( policy.zyppCommitPolicy().downloadMode() == DownloadOnly );
+  policy.summaryHints.clear();  // just in case ther's garbage from a previous use
   do
   {
     // CALL SOLVER
@@ -657,7 +730,7 @@ void solve_and_commit ( Zypper &zypper, SolveAndCommitPolicy policy )
         if ( success || SolverSettings::instance()._debugSolver )
           break;
 
-        success = show_problems( zypper );
+        success = show_problems( zypper, policy );
         if (!success)
         {
           zypper.setExitCode( ZYPPER_EXIT_ERR_ZYPP ); // bnc #242736
@@ -680,7 +753,7 @@ void solve_and_commit ( Zypper &zypper, SolveAndCommitPolicy policy )
 
     // SHOW SUMMARY
 
-    Summary summary( God->pool(), policy.summaryOptions() );
+    Summary summary( God->pool(), std::move(policy.summaryHints), policy.summaryOptions() );
 
     if ( zypper.out().verbosity() == Out::HIGH )
       summary.setViewOption( Summary::SHOW_VERSION );
@@ -696,6 +769,9 @@ void solve_and_commit ( Zypper &zypper, SolveAndCommitPolicy policy )
     if ( policy.zyppCommitPolicy().downloadMode() == DownloadOnly )
       summary.setDownloadOnly( true );
 
+    if ( policy.zyppCommitPolicy().dryRun() )
+      summary.setDryRun( true );
+
     // show the summary
     if ( zypper.out().type() == Out::TYPE_XML )
       summary.dumpAsXmlTo( cout );
@@ -762,12 +838,25 @@ void solve_and_commit ( Zypper &zypper, SolveAndCommitPolicy policy )
       // translators: help text for 'x' option in the 'Continue?' prompt
       // popts.setOptionHelp( 8, _("Explain why the packages are going to be installed.") );
 
-      std::string prompt_text( text::qContinue() );
+      std::string prompt_text;
+      {
+        str::Str str;
+        str << _("Backend") << ": ";
+        str << " " << ( policy.zyppCommitPolicy().singleTransModeEnabled() ? "single_rpmtrans" : "classic_rpmtrans" );
+        if ( policy.zyppCommitPolicy().dryRun() )
+          str << " " << "--dry-run";
+        if ( policy.zyppCommitPolicy().downloadMode() == DownloadOnly )
+          str << " " << "--download-only";
+        str << endl;
+        str << text::qContinue();
+        prompt_text = str;
+      }
 
       bool do_commit = false;
       unsigned reply;
       do
       {
+        zypper.out().gap();
         zypper.out().prompt( PROMPT_YN_INST_REMOVE_CONTINUE, prompt_text, popts );
         reply = get_prompt_reply( zypper, PROMPT_YN_INST_REMOVE_CONTINUE, popts );
 
